/*
 * Licensed Materials - Property of IBM                             
 *                                                              
 * (C) Copyright IBM Corp. 2006 All Rights Reserved.  
 */

package com.ibm.cn.wchao;

import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbService;
import com.ibm.broker.plugin.MbUserException;

/**
 * @author michael
 */
public abstract class EmailNode extends MbJavaComputeNode {

	/**
	 * Logs an error message to the platforms syslog/eventlog and throws a
	 * MbUserException contain the information. Throwing this exception will
	 * cause the broker to propagate the current message to the failure terminal
	 * 
	 * @param obj The source instance.
	 * @param methodName The method name.
	 * @param messageSrc The message source.
	 * @param key The message key.
	 * @param traceText The trace text, that will be written to user trace.
	 * @param inserts The inserts for the message.
	 * 
	 * @throws MbException
	 */
	protected void logErrorGenerateException(Object obj,
			String methodName,
			String messageSrc,
			String key,
			String traceText, 
			String[] inserts) throws MbException
			{
		MbService.logError(obj,
				methodName,
				messageSrc,
				key, 
				traceText,
				inserts);
		
		throw new MbUserException(obj,
				methodName,
				messageSrc,
				key, 
				traceText,
				inserts);
			}
	
	/**
	 * Logs an error message to the platforms syslog/eventlog and proceed processing
	 * without throwing any exception
	 * @param obj The source instance.
	 * @param methodName The method name.
	 * @param messageSrc The message source.
	 * @param key The message key.
	 * @param traceText The trace text, that will be written to user trace.
	 * @param inserts The inserts for the message.
	 */
	protected void logError(Object obj,
			String methodName,
			String messageSrc,
			String key,
			String traceText, 
			String[] inserts)
	{
		try {
			MbService.logError(obj,
					methodName,
					messageSrc,
					key, 
					traceText,
					inserts);
		} catch (MbException e) {
		}
	}
	/**
	 * Takes a Throwable object and writes the stacktrace to a String.
	 * This is used to provide that stacktrace as an insert to error
	 * messages.
	 * @param thr
	 * @return the stacktrace.
	 */
	protected String stackTraceToString(Throwable thr)
	{
		StackTraceElement[] elements = thr.getStackTrace();
		StringBuffer buf = new StringBuffer();
		
		for (int i = 0; i < elements.length; i++)
		{
			StackTraceElement element = elements[i];
			buf.append(element);
			
			if(i != elements.length-1)
				buf.append(System.getProperty("line.separator"));
		}
		
		return buf.toString();
	}
	
}
