/*
 * Licensed Materials - Property of IBM                             
 *                                                              
 * (C) Copyright IBM Corp. 2006 All Rights Reserved.  
 */

package com.ibm.cn.wchao;

import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.util.Iterator;
import java.util.ArrayList;

import javax.mail.Message;

import com.ibm.broker.plugin.MbBLOB;
import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;
import com.ibm.broker.plugin.MbRFH2C;
import com.ibm.broker.plugin.MbXML;
import com.ibm.broker.plugin.MbXMLNS;
import com.ibm.broker.plugin.MbXMLNSC;
import com.ibm.cn.wchao.POP3MailHelper.AttachmentInfo;

/**
 * @author michael
 */
public class EmailPOP3_JavaPopulate extends EmailNode {
	
	private String parserDomain = null;
	private String messageSet = null;
	private String messageFormat = null;
	private String messageType = null;
	private boolean processAttachments = false;
	private boolean processBody = true;
	
	private void populateSenderTree(MbElement element, String childElementName, String[] senders) throws MbException {
		MbElement childElement = element.createElementAsLastChild(MbElement.TYPE_NAME, childElementName, null);
		if (null != senders) for (int j = 0; j < senders.length; ++j)
			childElement.createElementAsLastChild(MbElement.TYPE_NAME, "InternetAddress", senders[j]);
	}

	/* (non-Javadoc)
	 * @see com.ibm.broker.javacompute.MbJavaComputeNode#evaluate(com.ibm.broker.plugin.MbMessageAssembly)
	 */
	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		
		parserDomain = (String) getUserDefinedAttribute("ParserDomain");
		messageSet = (String) getUserDefinedAttribute("MessageSet");
		messageType = (String) getUserDefinedAttribute("MessageType");
		messageFormat = (String) getUserDefinedAttribute("MessageFormat");
		processAttachments = ((Boolean)getUserDefinedAttribute("ProcessAttachments")).booleanValue();
		processBody = ((Boolean)getUserDefinedAttribute("ProcessBody")).booleanValue();
		if ("BLOB".equals(parserDomain)) {
			parserDomain = MbBLOB.PARSER_NAME;
			messageSet = null;
			messageType = null;
			messageFormat = null;
		} else if ("XML".equals(parserDomain)) {
			parserDomain = MbXML.PARSER_NAME;
			messageSet = null;
			messageType = null;
			messageFormat = null;
		} else if ("XMLNS".equals(parserDomain)) {
			parserDomain = MbXMLNS.PARSER_NAME;
			messageSet = null;
			messageType = null;
			messageFormat = null;
		} else if ("XMLNSC".equals(parserDomain)) {
			parserDomain = MbXMLNSC.PARSER_NAME;
			messageSet = null;
			messageType = null;
			messageFormat = null;
		} else if ("RFH2".equals(parserDomain)) {
			parserDomain = MbRFH2C.PARSER_NAME;
			messageSet = null;
			messageType = null;
			messageFormat = null;
		}
		MbMessage inMessage = inAssembly.getMessage();
		
		byte[] inBlob = (byte[]) inMessage.getRootElement().getLastChild().toBitstream(null, null, null, 0, 0, 0);
		String inText = new String(inBlob);

		InputStream bis = new BufferedInputStream(new ByteArrayInputStream(inText.getBytes()));
		Message message = POP3MailHelper.stream2Part(bis);
		
		MbMessageAssembly outAssembly = new MbMessageAssembly(
				inAssembly, new MbMessage(inAssembly
						.getLocalEnvironment()), inAssembly
						.getExceptionList(), new MbMessage(inAssembly.getMessage()));

		// Populate LocalEnvironment.Email.Recipients and
		// LocalEnvironment.Email.Senders
		MbElement localEnvRoot = outAssembly
				.getLocalEnvironment().getRootElement()
				.createElementAsFirstChild(
						MbElement.TYPE_NAME, "Email", null);
		MbElement recipientsElement = localEnvRoot
				.createElementAsLastChild(
						MbElement.TYPE_NAME, "Recipients",
						null);

		populateSenderTree(
				recipientsElement, "To",
				POP3MailHelper.getToRecipientsInternetAddress(message));
		populateSenderTree(
				recipientsElement, "Cc",
				POP3MailHelper.getCCRecipientsInternetAddress(message));
		populateSenderTree(
				recipientsElement, "Bcc",
				POP3MailHelper.getBCCRecipientsInternetAddress(message));
		populateSenderTree(
				localEnvRoot, "Senders",
				POP3MailHelper.getSenderInternetAddress(message));

		// Populate LocalEnvironment.Email.Subject
		localEnvRoot.createElementAsLastChild(
				MbElement.TYPE_NAME, "Subject",
				POP3MailHelper.getSubject(message));

		MbOutputTerminal out = getOutputTerminal("out");
		
		ArrayList<Object> msgList = new ArrayList<Object>();
		if (processBody) {
			String body = POP3MailHelper.getBody(message);
			msgList.add(body);
		}
		if (processAttachments) {
			AttachmentInfo[] attachments = POP3MailHelper.getAttachments(message);
			if (null != attachments) for (int i = 0; i < attachments.length; ++i)
				msgList.add(attachments[i]);
		}
		Iterator iter = msgList.iterator();
		while (iter.hasNext()) {
			// Create new message
			MbMessage outMessage = outAssembly.getMessage();
			
			String text = null;
			Object msg = iter.next();
			if (msg instanceof String) {
				text = (String) msg;
			} else if (msg instanceof AttachmentInfo) {
				text = ((AttachmentInfo)msg).attachment;
				// Populate LocalEnvironment.Email.Filename
				MbElement fnElem = localEnvRoot.getFirstElementByPath("Filename");
				if (fnElem == null) {
					localEnvRoot.createElementAsLastChild(
						MbElement.TYPE_NAME, "Filename",
						((AttachmentInfo)msg).fileName);
				} else {
					fnElem.setValue(((AttachmentInfo)msg).fileName);
				}
			}
			// Replace the last child with the email body as
			// bitstream
			try {
				outMessage.getRootElement().getLastChild().detach();
					
				outMessage.getRootElement()
					.createElementAsLastChildFromBitstream(
						text.getBytes(), parserDomain,
						messageType, messageSet, messageFormat, 0, 0, 0);
					
				// Propagate to next node in the flow
				out.propagate(outAssembly);
			} finally {
				// clear the outMessage
				// outMessage.clearMessage();
			}
		}
	}

}
