/*
 * Licensed Materials - Property of IBM                             
 *                                                              
 * (C) Copyright IBM Corp. 2006 All Rights Reserved.  
 */

package com.ibm.cn.wchao;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.activation.CommandMap;
import javax.activation.MailcapCommandMap;
import javax.mail.Address;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;



/**
 * @author michael
 */
public class POP3MailHelper {

	public static class AttachmentInfo {
		public String fileName = null;
		public String attachment = null;
	}
	
	private Store store = null;
    private Folder folder = null;
    
    // Work-around with a problem of Java Activation Framework: the mailcap file in activation.jar sometimes cannot be seen correctly
    static {
		MailcapCommandMap mc = (MailcapCommandMap)CommandMap.getDefaultCommandMap();
		mc.addMailcap("text/html;; x-java-content-handler=com.sun.mail.handlers.text_html");
		mc.addMailcap("text/plain;; x-java-content-handler=com.sun.mail.handlers.text_plain");
		mc.addMailcap("multipart/*;; x-java-content-handler=com.sun.mail.handlers.multipart_mixed");
		CommandMap.setDefaultCommandMap(mc);
    }

    public void connect(String popServer, String popUser, String popPassword, String folderName, boolean readOnly) {
	    try {
			Properties props = System.getProperties();
			Session session = Session.getDefaultInstance(props, null);
			store = session.getStore("pop3");
			store.connect(popServer, popUser, popPassword);
			folder = store.getDefaultFolder();
			if (folder == null)
				throw new Exception("No default folder");
			folder = folder.getFolder(folderName);
			if (folder == null)
				throw new Exception("No POP3 INBOX");
			folder.open((readOnly ? Folder.READ_ONLY : Folder.READ_WRITE));
		} catch (Exception ex) {
			ex.printStackTrace();
			try {
				if (store != null)
					store.close();
			} catch (Exception ex2) {
				ex2.printStackTrace();
			}
		}
    }
    
    public void disconnect() {
    	try {
    		if (folder != null) {
    			folder.close(true);
    		}
    		if (store != null) store.close();
    	} catch (Exception ex) {
    		ex.printStackTrace();
    	}
    }
    
    public boolean isConnected() {
    	return store != null && store.isConnected();
    }
    
    public boolean isOpen() {
    	return folder != null && folder.isOpen();
    }
    
    public static Session getSession() {
    	return Session.getDefaultInstance(System.getProperties());
    }
    
	public Message[] receive() {
        Message[] messages = null;
        try {
        	if (folder != null)
        		messages = folder.getMessages();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        return messages;
	}
	
	public static String[] getSenderInternetAddress(Message message) {
		String fromsRet[] = null;
		try {
			Address[] froms = message.getFrom();
			fromsRet = new String[froms.length];
			for (int i = 0; i < froms.length; ++i)
				fromsRet[i] = ((InternetAddress)froms[i]).getAddress();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		return fromsRet;
	}
	
	public static String[] getToRecipientsInternetAddress(Message message) {
		String tosRet[] = null;
		try {
			Address[] tos = message.getRecipients(Message.RecipientType.TO);
			if (null != tos) {
				tosRet = new String[tos.length];
				for (int i = 0; i < tos.length; ++i)
					tosRet[i] = ((InternetAddress)tos[i]).getAddress();
			}
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		return tosRet;
	}
	
	public static String[] getCCRecipientsInternetAddress(Message message) {
		String ccsRet[] = null;
		try {
			Address[] ccs = message.getRecipients(Message.RecipientType.CC);
			if (null != ccs) {
				ccsRet = new String[ccs.length];
				for (int i = 0; i < ccs.length; ++i)
					ccsRet[i] = ((InternetAddress)ccs[i]).getAddress();
			}
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		return ccsRet;
	}
	
	public static String[] getBCCRecipientsInternetAddress(Message message) {
		String bccsRet[] = null;
		try {
			Address[] bccs = message.getRecipients(Message.RecipientType.BCC);
			if (null != bccs) {
				bccsRet = new String[bccs.length];
				for (int i = 0; i < bccs.length; ++i)
					bccsRet[i] = ((InternetAddress)bccs[i]).getAddress();
			}
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		return bccsRet;
	}
	
	public static Part[] getChildParts(Part part) {
		Object content;
		
		try {
			int count = 1;
			content = part.getContent();
			if (part.isMimeType("multipart/*")) {
				count = ((Multipart)content).getCount();
			}
			Part[] ret = new Part[count];
			ret[0] = part;
			for (int i = 1; i < count; ++i)
				ret[i] = ((Multipart)content).getBodyPart(i);
			return ret;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static void msg2Stream(Part part, OutputStream stream) {
		try {
			part.writeTo(stream);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
	}

	public static String getBody(Message message) {
		String body = null;
		Part part = message;
		try {
			Object content = part.getContent();

			if (part.isMimeType("multipart/*")) {
				part = ((Multipart)content).getBodyPart(0);
			}
			if (part.isMimeType("text/plain") || part.isMimeType("text/html")) {
				InputStream is = part.getInputStream();
				BufferedReader reader = new BufferedReader(new InputStreamReader(is));
				int c;
                StringBuffer msg = new StringBuffer();
                while((c=reader.read())!=-1)
                    msg.append((char)c);
                body = msg.toString();
                reader.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		return body;
	}
	
	public static String getSubject(Message message) {
		try {
			return message.getSubject();
		} catch (MessagingException e) {
			return null;
		}
	}
	
	public static String getIntendedRecipient(Message message) {
		try {
			String[] headers = message.getHeader("Delivered-To");
			if (headers != null && headers.length > 0)
				return headers[0];
		} catch (MessagingException e) {
		}
		return null;
	}

	public static AttachmentInfo[] getAttachments(Message message) {
		List<AttachmentInfo> ret = new ArrayList<AttachmentInfo>();
		try {
			Object content = message.getContent();
			if (!(content instanceof Multipart))	// must be Multpart for attachments
				return null;
			for (int i = 1; i < ((Multipart)content).getCount(); ++i) {
				Part part = ((Multipart)content).getBodyPart(i);
				String disposition = part.getDisposition();
				if (null != disposition && (Part.ATTACHMENT.endsWith(disposition) || Part.INLINE.endsWith(disposition))) {
					AttachmentInfo ainfo = new AttachmentInfo();
					ainfo.fileName = part.getFileName();
					InputStream is = part.getInputStream();
					BufferedReader reader = new BufferedReader(new InputStreamReader(is));
					int c;
	                StringBuffer msg = new StringBuffer();
	                while((c=reader.read())!=-1)
	                    msg.append((char)c);
	                ainfo.attachment = msg.toString();
	                reader.close();
	                ret.add(ainfo);
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (MessagingException e) {
			e.printStackTrace();
		}
		AttachmentInfo[] retS = new AttachmentInfo[ret.size()];
		ret.toArray(retS);
		return retS;
	}

	public static MimeMessage stream2Part(InputStream stream) {
		try {
			return new MimeMessage(getSession(), stream);
		} catch (MessagingException e) {
			e.printStackTrace();
			return null;
		}
	}

}
